command = xplor.command
from jCoupPot import JCoupPot 
from noePot import NOEPot

import prePot

from xplor import select

from xplorPot import XplorPot
from rdcPotTools import *
from pdbTool import *
from atomAction import *
from selectTools import *
from simulationTools import *
from ivm import IVM

import protocol
import monteCarlo

protocol.initRandomSeed()

protocol.initParams('../parallhdg_new.pro')

protocol.initStruct('./3eza_AH.psf')
protocol.initCoords('./3eza_AH.pdb')

AtomSel("resid 21:147 or (resi 20 and (name c or name o)) or (resi 148 and (name hn or name n))").apply(SetProperty('segmentName','ALT0'))

numT=2

for cnt in range(1,numT):
  seg="ALT%d" % cnt
  command("duplicate sele=(segid ALT0) segid=%s end" % seg)
  pass

#protocol.initCoords('Optimize_0612/cam1_112.pdb')

protocol.initNBond(repel=1.2)

command("""

    constraints

    inter = (segid ALT1 and (resid 24:142)) (segid " ")

    inter = (segid ALT1 and (resid 20:23 or resid 143:148)) (segid ALT1 or segid " ")

    weights * 1 end end

    """)

#
# annealing settings
#

init_t  = 3000
final_t = 25

cool_steps = 12000

from simulationTools import MultRamp, StaticRamp, InitialParams
rampedParams=[]

potList = PotList()
potList.add( XplorPot("BOND") )

potList.add( XplorPot("ANGL") )
rampedParams.append( MultRamp(0.4,1,"potList['ANGL'].setScale(VALUE)") )

potList.add( XplorPot("IMPR") )
rampedParams.append( MultRamp(0.4,1,"potList['IMPR'].setScale(VALUE)") )

potList.add( XplorPot("VDW") )
rampedParams.append( MultRamp(1.2,0.8,
                              "command('param nbonds repel VALUE end end')") )
rampedParams.append( MultRamp(.004,4,
                              "command('param nbonds rcon VALUE end end')") )

#potList.add( XplorPot("COLL") )

#from gyrPotTools import create_GyrPot
#gyr = create_GyrPot('Vgyr','not (segid alt0 or resi 232:250)')
#potList.append(gyr)

#protocol.initRamaDatabase('protein')
#potList.append( XplorPot('RAMA') )
#rampedParams.append( MultRamp(.002,1,"potList['RAMA'].setScale(VALUE)") )

from torsionDBPotTools import create_TorsionDBPot
torsionDBPot = create_TorsionDBPot('tDB', selection='segid alt1 and (resid 20:23 or resid 143:148)')
potList.append( torsionDBPot )
rampedParams.append( MultRamp(.01,1,"torsionDBPot.setScale(VALUE)") )

noe=PotList('noe')
potList.append(noe)
from noePotTools import create_NOEPot

pot = create_NOEPot('cxms',"./noe_inter5_4.tbl")
pot.setPotType("hard")
pot.setScale(1)       
pot.setAveType("sum")
noe.append(pot)

rampedParams.append( MultRamp(1,100, "noe.setScale( VALUE )") )

# IVM setup
#

dyn  = IVM()

dyn.fix(""" segid ALT0 or segid "  " """)

# 24 phi and 142 psi

dyn.group(""" segid ALT1 and resid 24:142 """)


# linker 23, 144-146
# 20 psi and 148 phi

dyn.breakAllBondsIn("""(resi 20 and name CA) or (resi 20 and name C)""")
dyn.breakAllBondsIn("""(resi 148 and name N) or (resi 148 and name CA)""")

#dyn.breakAllBondsTo(""" segi alt1 and resi 145 and name cb """)
#dyn.breakAllBondsTo(""" segi alt1 and resi 147 and name ca """)

protocol.updatePseudoAtoms()

dyn.autoTorsion()

def structLoopAction(loopInfo):

    protocol.initMinimize(dyn, potList=potList)
    potList["VDW"].setScale(0)
    InitialParams( rampedParams )
    dyn.run()
    dyn.run()

    #
    # high temp dynamics
    #

    ini_timestep = 0.010
    protocol.initDynamics(dyn,
                          potList=potList,
                          bathTemp=init_t,
                          initVelocities=True,
                          stepsize=ini_timestep,
                          finalTime=20,
                          printInterval=100)
    dyn.run()

    #
    # cooling
    #

    timestep=ini_timestep
    potList["VDW"].setScale(1)

    protocol.initDynamics(dyn,
                          potList=potList,
                          bathTemp=init_t,
                          initVelocities=True,
                          stepsize=timestep,
                          finalTime=0.5,
                          printInterval=100)

    dyn.setResetCMInterval( 100 )

    AnnealIVM(initTemp =init_t,
              finalTemp=final_t,
              numSteps = 50,
              ivm=dyn,
              rampedParams = rampedParams).run()
    #
    # final Powell minimization
    #
    protocol.initMinimize(dyn)
    dyn.run()

    loopInfo.writeStructure(potList)

    pass

StructureLoop(numStructures=256,
              pdbTemplate='./Optimize_1220/cam25_STRUCTURE.pdb',
              averagePotList=potList,
              genViolationStats=1,
              structLoopAction=structLoopAction).run()

